import stat

import rpm

from Testing import get_tested_mock_package


UnexpandedMacroFilesPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/%{unexpanded}/test': {},
        '/usr/bin/unexpanded-macro-files': {'is_dir': True},
        '/usr/share/licenses/unexpanded-macro-files': {'is_dir': True},
        '/usr/share/licenses/unexpanded-macro-files/LICENSE': {},
    },
    header={'requires': []},
)


Python3PowerPackage = get_tested_mock_package(
    name='python3-power',
    lazyload=True,
    header={'requires': ['python(abi) = 3.3']},
    files={
        '/usr/lib/python3.3/site-packages/power/__init__.py': {
            'content-path': 'files/python3-power/__init__.py',
            'metadata': {'mtime': 1_363_887_182},
        },
        '/usr/lib/python3.3/site-packages/power/common.py': {
            'content-path': 'files/python3-power/common.py',
            'metadata': {'mtime': 1_363_887_182},
        },
        '/usr/lib/python3.3/site-packages/power/__pycache__/__init__.cpython-33.pyc': {
            'content-path': 'files/python3-power/__init__.cpython-33.pyc',
        },
        '/usr/lib/python3.3/site-packages/power/__pycache__/__init__.cpython-33.pyo': {
            'content-path': 'files/python3-power/__init__.cpython-33.pyo',
        },
        '/usr/lib/python3.3/site-packages/power/__pycache__/common.cpython-33.pyc': {
            'content-path': 'files/python3-power/common.cpython-33.pyc',
        },
        '/usr/lib/python3.3/site-packages/power/__pycache__/common.cpython-33.pyo': {
            'content-path': 'files/python3-power/common.cpython-33.pyo',
        },
        '/usr/share/doc/python3-power-1.1/index.html': {
            'metadata': {'flags': rpm.RPMFILE_DOC, 'size': 100},
        },
    },
)


Python3PowerBrokenPackage = Python3PowerPackage.clone(
    files={
        '/usr/lib/python3.3/site-packages/power/__init__.py': {
            'content-path': 'files/python3-power/__init__.py',
            'metadata': {'mtime': 1_363_887_182},
        },
        '/usr/lib/python3.3/site-packages/power/__pycache__/__init__.cpython-33.pyc': {
            'content': 'BROKEN-MAGIC-PYC',
        },
        '/usr/share/doc/python3-power-1.1/index.html': {
            'metadata': {'flags': rpm.RPMFILE_DOC, 'size': 100},
        },
    },
)


TestDocumentationPackage = get_tested_mock_package(
    name='testdocumentation',
    lazyload=True,
    files={
        'usr/share/doc/packages/testdocumentation': {'is_dir': True},
        '/usr/share/doc/packages/testdocumentation/README1.gz': {
            'content-path': 'files/README1.gz',
            'metadata': {'flags': rpm.RPMFILE_DOC},
        },
        '/usr/share/doc/packages/testdocumentation/README2.bz2': {
            'content-path': 'files/README2.bz2',
            'metadata': {'flags': rpm.RPMFILE_DOC},
        },
        '/usr/share/doc/packages/testdocumentation/README3.xz': {
            'content-path': 'files/README3.xz',
            'metadata': {'flags': rpm.RPMFILE_DOC},
        },
    },
    header={'requires': {}},
)


NetmaskDebugsourcePackage = get_tested_mock_package(
    lazyload=True,
    name='netmask-debugsource',
    files={
        '/usr/src/debug/netmask-2.4.3-5.fc27.x86_64/errors.c': {
            'content-path': 'files/netmask/errors.c',
        },
        '/usr/src/debug/netmask-2.4.3-5.fc27.x86_64/errors.h': {
            'content-path': 'files/netmask/errors.h',
        },
        '/usr/src/debug/netmask-2.4.3-5.fc27.x86_64/main.c': {
            'content-path': 'files/netmask/main.c',
        },
        '/usr/src/debug/netmask-2.4.3-5.fc27.x86_64/netmask.c': {
            'content-path': 'files/netmask/netmask.c',
        },
        '/usr/src/debug/netmask-2.4.3-5.fc27.x86_64/netmask.h': {
            'content-path': 'files/netmask/netmask.h',
        },
    },
    header={
        'version': '2.4.3',
        'release': '5.fc27',
        'requires': [
            'rpmlib(CompressedFileNames) <= 3.0.4-1',
            'rpmlib(FileDigests) <= 4.6.0-1',
            'rpmlib(PayloadFilesHavePrefix) <= 4.0-1',
            'rpmlib(PayloadIsXz) <= 5.2-1',
        ]
    }
)


IncorrectFSFAddress = get_tested_mock_package(
    lazyload=True,
    name='incorrectfsf',
    files={
        '/usr/incorrect-fsf.txt': {
            'content-path': 'files/incorrect-fsf.txt',
        },
        '/usr/correct-fsf.txt': {
            'content-path': 'files/correct-fsf.txt',
        },
    },
    header={
        'version': '2.4.3',
        'release': '5.fc27',
        'requires': [],
    },
)


MakefileJunkPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/CMakeLists.txt': {},
        '/usr/share/Makefile.am': {'metadata': {'flags': rpm.RPMFILE_DOC}},
        '/usr/share/Makefile.in': {},
        '/usr/share/selinux': {'is_dir': True},
        '/usr/share/selinux/Makefile': {},
        '/usr/src/foo': {'is_dir': True},
        '/usr/src/foo/Makefile': {}
    },
    header={'requires': []},
)

SphinxInvPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/doc/packages/python3-greenlet/html/objects.inv': {
            'metadata': {'flags': rpm.RPMFILE_DOC},
            'content': '# Sphinx inventory version 2',
        },
    },
    header={'requires': []},
)


FileChecksPackage = get_tested_mock_package(
    lazyload=True,
    files=[
        '/run/foo',
        '/etc/systemd/system/foo',
        '/etc/udev/rules.d/foo',
        '/etc/tmpfiles.d/foo',
        '/bin/foo/bar',
        '/site_perl/foo',
        '/~backup.rej',
        '/.gitignore',
        '/.htaccess',
        '/usr/share/doc/perl-foo/MANIFEST',
        '/usr/info/dir',
    ],
    header={'requires': []},
)


TclPackage = get_tested_mock_package(
    lazyload=True,
    files=['/usr/lib64/tcl/pkgIndex.tcl'],
    header={'requires': []},
)


RustFilesPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/bar.rs': {
            'content': '#![allow(box_pointers)]',
            'metadata': {'mode': 0o644 | stat.S_IFREG},
        },
        '/etc/foo.rs': {
            'content': '#![allow(box_pointers)]',
            'metadata': {'mode': 0o755 | stat.S_IFREG},
        },
    },
    header={'requires': []},
)


ManPagesPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/etc/ngircd.conf': {'content': 'conf'},
        '/usr/sbin/ngircd': {'content': 'XXXX', 'metadata': {'mode': 0o755 | stat.S_IFREG}},
        '/usr/share/man/man5/ngircd.conf.5.gz': {'content': 'gz'},
        '/usr/share/man/man8/ngircd.8.gz': {'content': 'gz', 'metadata': {'flags': rpm.RPMFILE_DOC}},
    },
    header={'requires': []},
)


DevelopmentPackage = get_tested_mock_package(
    lazyload=True,
    name='my-package-devel',
    files={
        '/usr/x.typelib': {'metadata': {'size': 100}},
    },
    header={'requires': []},
)


Shlib1Package = get_tested_mock_package(
    name='shlib1',
    lazyload=True,
    files={
        '/usr/lib/libfoo-2.so': {
            'content-path': 'files/shlib2/libfoo-2.so',
            'metadata': {'mode': 0o755 | stat.S_IFREG},
        },
        '/usr/lib/libfoo-2.so.foo': {
            'content-path': 'files/shlib2/libfoo-2.so.foo',
            'metadata': {'mode': 0o644 | stat.S_IFREG},
        },
        '/usr/lib/libfoo.so': {
            'linkto': 'libfoo.so.1',
            'metadata': {'mode': 0o777 | stat.S_IFREG},
        },
        '/usr/lib/libfoo.so.1': {
            'content-path': 'files/shlib2/libfoo.so.1',
            'metadata': {'mode': 0o755 | stat.S_IFREG},
        },
    },
    header={'requires': []},
)
Shlib2DevelPackage = Shlib1Package.clone(name='shlib2-devel')


FileZeroLengthPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/lib/python': {'is_dir': True},
        # Ok files
        '/etc/security/console.apps/myapp': {
            'create_dirs': True,
            'metadata': {
                'mode': 0o644 | stat.S_IFREG,
                'size': 100,
                'flags': rpm.RPMFILE_CONFIG,
            },
        },
        '/etc/share/doc/packages/file-zero-length/dummydoc': {
            'create_dirs': True,
            'metadata': {
                'mode': 0o644 | stat.S_IFREG,
                'size': 100,
                'flags': rpm.RPMFILE_DOC,
            },
        },
        '/usr/lib/nonemptyfile': {
            'metadata': {'mode': 0o644 | stat.S_IFREG, 'size': 100},
        },
        # Not Ok file
        '/usr/lib/emptyfile': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
        # Ignored by normal_zero_length_regex
        '/usr/lib/.nosearch': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
        '/usr/lib/python/__init__.py': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
        '/usr/lib/python/py.typed': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
        '/usr/lib/python/pypackagefromwheel-0.0.0.dist-info/REQUESTED': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
        '/usr/lib/ruby/gem.build_complete': {'metadata': {'mode': 0o644 | stat.S_IFREG}},
    },
    header={'requires': []},
)


ManualPagesPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/man/man0p': {'is_dir': True},
        '/usr/share/man/man0p/foo.3.gz': {},
        '/usr/share/man/man1': {'is_dir': True},
        '/usr/share/man/man1/test.1.zst': {},
        '/usr/share/man/man3': {'is_dir': True},
        '/usr/share/man/man3/foo': {'is_dir': True},
        '/usr/share/man/man3/foo/bar': {'is_dir': True},
        '/usr/share/man/man3/foo/bar/baz.3.gz': {},
        '/usr/share/man/man3/some.3pm.gz': {}
    },
    header={'requires': []},
)


PythonShebangLinkPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/package/bin.py': {
            'content': '#!/usr/bin/python3\nprint("python required")',
            'metadata': {'mode': 0o755 | stat.S_IFREG},
        },
        '/usr/bin/testlink': {
            'linkto': '../share/package/bin.py',
        },
    },
    header={},
)
PythonShebangLinkOkPackage = PythonShebangLinkPackage.clone(
    header={'requires': ['/usr/bin/python3']},
)


DirectoryWithoutXPermPackage = get_tested_mock_package(
    lazyload=True,
    header={'requires': []},
    files={
        '/etc/raddb/mods-config/sql/moonshot-targeted-ids/mysql': {
            'is_dir': True,
            'metadata': {'mode': 0o640 | stat.S_IFDIR},
        },
        '/etc/raddb/mods-config/sql/moonshot-targeted-ids/postgresql': {
            'is_dir': True,
            'metadata': {'mode': 0o640 | stat.S_IFDIR},
        },
        '/etc/raddb/mods-config/sql/moonshot-targeted-ids/sqlite': {
            'is_dir': True,
            'metadata': {'mode': 0o640 | stat.S_IFDIR},
        },
    },
)


DirectoryWithoutXPerm2Package = get_tested_mock_package(
    lazyload=True,
    header={'requires': []},
    files={
        '/etc/raddb': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/certs': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/mods-available': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/mods-config': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/policy.d': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/sites-available': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/etc/raddb/sites-enabled': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/usr/lib64/freeradius': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
        '/usr/share/freeradius': {'is_dir': True, 'metadata': {'mode': 0o640 | stat.S_IFDIR}},
    },
)


FilesWithoutPermsPackage = get_tested_mock_package(
    lazyload=True,
    header={'requires': []},
    files={
        '/var/lib/pipewire': {'is_dir': True, 'metadata': {'mode': 0o000 | stat.S_IFDIR}},
        '/var/lib/dir_read': {'is_dir': True, 'metadata': {'mode': 0o755 | stat.S_IFDIR}},
        '/var/lib/pipewire/ghost_file': {'metadata': {'mode': 0o000, 'flags': rpm.RPMFILE_GHOST}},
        '/var/lib/pipewire/ghost_file_read': {'metadata': {'mode': 0o644, 'flags': rpm.RPMFILE_GHOST}},
        '/var/lib/pipewire/normal_file': {'metadata': {'mode': 0o000}},
        '/var/lib/pipewire/normal_file_read': {'metadata': {'mode': 0o644}},
    },
)


FilesWithoutPermsTmpfilesPackage = get_tested_mock_package(
    lazyload=True,
    header={'requires': []},
    files={
        '/run/netconfig/resolv.conf': {'metadata': {'mode': 0o000, 'flags': rpm.RPMFILE_GHOST}},
        '/run/netconfig/yp.conf': {'metadata': {'mode': 0o644, 'flags': rpm.RPMFILE_GHOST}},
        '/run/netconfig': {'is_dir': True, 'metadata': {'mode': 0o000, 'flags': rpm.RPMFILE_GHOST | stat.S_IFDIR}},
        '/usr/lib/tmpfiles.d/netconfig.conf': {
            'content': """
d /run/netconfig 0755 root group -
f /run/netconfig/resolv.conf 0644 root root -
f /run/netconfig/yp.conf 0644 root root -
"""
        },
    },
)


# Package with just a ghost file that doesn't exists in rpmroot during build so
# it has no permissions
NonReadableGhostPackage = get_tested_mock_package(
    lazyload=True,
    header={'requires': []},
    files={
        '/boohoo': {
            'metadata': {
                'mode': 0o000 | stat.S_IFREG,
                'flags': rpm.RPMFILE_GHOST,
            },
        },
    },
)



#!/usr/bin/python3
""" This script is used to generate values for
    config/Fedora/configs/Fedora/users-groups.toml
"""

import datetime
import os
import re

users = set()
groups = set()
uidgid_regex = re.compile(r'^\s*(\S+)\s+(-|\d+)\s+(-|\d+|\(\d+\))\s')
for uidgid_file in ['/usr/share/doc/setup/uidgid']:
    if os.path.exists(uidgid_file):
        with open(uidgid_file) as fobj:
            try:
                for line in fobj.read().strip().splitlines():
                    if line.startswith('#'):
                        continue
                    res = uidgid_regex.search(line)
                    if res:
                        name = res.group(1)
                        if res.group(2) != '-':
                            users.add(name)
                        if res.group(3) != '-' and '(' not in res.group(3):
                            groups.add(name)
                    del res
                del line
            finally:
                fobj.close()
            del fobj

name = '/'.join(__file__.split('/')[-2:])
date = datetime.datetime.utcnow().strftime('%Y-%m-%d')
print(f'# generated by {name} on {date}\n')

print(f'StandardUsers = {sorted(users)}\n')

print(f'StandardGroups = {sorted(groups)}')



from importlib.metadata import distribution

from mockdata.mock_erlang import ErlangPackage
from packaging.version import parse
import pytest
from rpmlint.checks.ErlangCheck import ErlangCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def erlangcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = ErlangCheck(CONFIG, output)
    return output, test


@pytest.mark.skipif(parse(distribution('pybeam').version) < parse('0.7'), reason='pybeam >= 0.7 required')
@pytest.mark.parametrize('package', [ErlangPackage])
def test_erlang(package, erlangcheck):
    output, test = erlangcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: beam-compiled-without-debuginfo /usr/lib/erlang/m.beam' in out
    assert 'W: beam-compile-info-missed /usr/lib/erlang/m-no-CInf.beam' in out



from mockdata.mock_icon_sizes import WrongIconSizePackage
import pytest
from rpmlint.checks.IconSizesCheck import IconSizesCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def iconsizescheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = IconSizesCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [WrongIconSizePackage])
def test_icon_sizes(package, iconsizescheck):
    output, test = iconsizescheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: wrong-icon-size /usr/share/tasque/icons/hicolor/16x16/status/tasque-note.png expected: 16x16 actual: 22x22' in out



from Testing import get_tested_mock_package


Libtest1Package = get_tested_mock_package(
    lazyload=True,
    name='libtest1',
    files={
        '/usr/lib64/libtest.so.1.5.0': {
            # SONAME: libtest.so.1x
            'content-path': 'files/shlib_policy/libtest.so.1.5.0',
        }
    },
    header={
        'requires': [
            '/sbin/ldconfig',
            'libc.so.6()(64bit)',
            'libc.so.6(GLIBC_2.2.5)(64bit)',
        ]
    }
)


LibslpMissingSuffixPackage = get_tested_mock_package(
    lazyload=True,
    name='libslp-missing-suffix',
    files={
        '/usr/lib64/hello': {'is_dir': True},
        '/usr/lib64/hello/libhello2.so': {'content-path': 'files/shlib_policy/libhello2.so'},
        '/usr/lib64/libhello.so': {'content-path': 'files/shlib_policy/libhello.so'},
        '/usr/lib64/libhello2.so': {'content-path': 'files/shlib_policy/libhello2.so'}
    },
    header={
        'requires': [
            'libc.so.6()(64bit)',
            'libc.so.6(GLIBC_2.2.5)(64bit)',
            # Excessive dependency
            'libsparta.so.2',
        ]
    }
)


Libslp1234Package = get_tested_mock_package(
    lazyload=True,
    name='libslp1234',
    files={
        '/usr/lib64/hello': {'is_dir': True},
        '/usr/lib64/hello/libhello2.so': {'content-path': 'files/shlib_policy/libhello2.so'},
        '/usr/lib64/libhello.so': {'content-path': 'files/shlib_policy/libhello.so'},
        '/usr/lib64/libhello2.so': {'content-path': 'files/shlib_policy/libhello2.so'}
    },
    header={
        'requires': [
            'libc.so.6()(64bit)',
            'libc.so.6(GLIBC_2.2.5)(64bit)',
            # Fixed shlib dependency
            'libsparta.so.2 = 1.23',
        ]
    }
)



from mockdata.mock_doc import (
    DocFileDependencyPackage,
    InstallFileInDocPackage,
    MyDocPackage,
)
import pytest
from rpmlint.checks.DocCheck import DocCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def doccheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = DocCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [MyDocPackage])
def test_doccheck(package, doccheck):
    output, test = doccheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: executable-docs /usr/share/doc/packages/mydoc/doc.html' in out
    assert 'E: executable-docs /usr/share/doc/packages/mydoc/README' in out
    assert 'W: package-with-huge-docs 100%' in out


@pytest.mark.parametrize('package', [DocFileDependencyPackage])
def test_doc_file_dep(package, doccheck):
    output, test = doccheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: doc-file-dependency' in out
    assert 'W: install-file-in-docs' not in out


@pytest.mark.parametrize('package', [InstallFileInDocPackage])
def test_install_file_in_docs(package, doccheck):
    output, test = doccheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: install-file-in-docs' in out
    assert 'E: executable-docs' not in out
    assert 'W: package-with-huge-docs 100%' not in out
    assert 'W: doc-file-dependency' not in out



import glob
import os
from pathlib import Path
import platform
import re
import shutil
import subprocess

from rpmlint.config import Config
from rpmlint.pkg import FakePkg, Pkg
import rpmlint.spellcheck


def _testpath():
    return Path(os.environ.get('TESTPATH', Path(__file__).parent))


TEST_CONFIG = [_testpath() / 'configs/test.config']
CONFIG = Config(TEST_CONFIG)

# predicates used for pytest.mark.skipif decorators
IS_X86_64 = platform.machine() == 'x86_64'
IS_I686 = re.match(platform.machine(), 'i[3456]86')
HAS_32BIT_GLIBC = glob.glob('/lib/ld-linux.so.*')
HAS_CHECKBASHISMS = shutil.which('checkbashisms')
HAS_DASH = shutil.which('dash')
HAS_DESKTOP_FILE_UTILS = shutil.which('desktop-file-validate')
HAS_APPSTREAM_GLIB = shutil.which('appstream-util')

RPMDB_PATH = subprocess.run(['rpm', '--eval', '%_dbpath'], encoding='utf8', stdout=subprocess.PIPE).stdout
HAS_RPMDB = RPMDB_PATH and Path(RPMDB_PATH.strip()).exists()


def _has_dictionary(language):
    if not rpmlint.spellcheck.ENCHANT:
        return False
    spell = rpmlint.spellcheck.Spellcheck()
    spell._init_checker(language)
    return spell._enchant_checkers.get(language)


HAS_ENGLISH_DICTIONARY = _has_dictionary('en_US')
HAS_CZECH_DICTIONARY = _has_dictionary('cs_CZ')
HAS_FRENCH_DICTIONARY = _has_dictionary('fr')


def get_tested_path(*paths):
    return _testpath().joinpath(*paths)


def get_tested_package(name, testdir):
    filename = Path(name).name + '-*.rpm'
    candidates = list(get_tested_path(name).parent.glob(filename))
    assert len(candidates) == 1
    return Pkg(candidates[0], testdir)


def get_tested_spec_package(name):
    filename = Path(name).name + '.spec'
    candidates = list(get_tested_path(name).parent.glob(filename))
    assert len(candidates) == 1
    return FakePkg(candidates[0])


class LazyMock:
    """
    Class to store mock package definition and create the actual mock package
    when needed, when some internal attribute or method is requested.
    """

    def __init__(self, files, header, name):
        self._lazy_pkg = None
        self._lazy_files = files
        self._lazy_header = header
        self._lazy_name = name

    @property
    def _fake_pkg(self):
        if not self._lazy_pkg:
            self._lazy_pkg = get_tested_mock_package(self._lazy_files,
                                                     self._lazy_header,
                                                     self._lazy_name)
        return self._lazy_pkg

    def clone(self, files=None, header=None, name=None, extend=False):
        """
        Copies this LazyMock modifying some properties
        """

        if files is None:
            files = self._lazy_files
        if header is None:
            header = self._lazy_header
        elif extend:
            header = self._lazy_header | header
        if name is None:
            name = self._lazy_name

        return LazyMock(files, header, name)

    def __getitem__(self, key):
        return self._fake_pkg.__getitem__(key)

    def __getattr__(self, name):
        return getattr(self._fake_pkg, name)


def get_tested_mock_package(files=None, header=None, name='mockPkg', lazyload=False):
    if lazyload:
        return LazyMock(files, header, name)

    mockPkg = FakePkg(name)
    if files is not None:
        if isinstance(files, dict):
            # full path for test files
            for attrs in files.values():
                if 'content-path' in attrs:
                    attrs['content-path'] = get_tested_path(attrs['content-path'])

        mockPkg.create_files(files)
    if header is not None:
        mockPkg.add_header(header)
    mockPkg.initiate_files_base_data()
    return mockPkg



import re

from rpmlint.checks.AbstractCheck import AbstractCheck


class IconSizesCheck(AbstractCheck):
    file_size_regex = re.compile(r'/icons/[^/]+/(?P<x>\d+)x(?P<y>\d+)/')
    info_size_regex = re.compile(r'(?P<x>\d+) x (?P<y>\d+)')

    def check(self, pkg):
        if pkg.is_source:
            return

        for fname, pkgfile in pkg.files.items():
            if '/animations/' in fname:
                continue

            res = self.file_size_regex.search(fname)
            res2 = self.info_size_regex.search(pkgfile.magic)
            if res and res2:
                sizes = (res.group('x'), res.group('y'))
                actualsizes = (res2.group('x'), res2.group('y'))
                if abs(int(sizes[0]) - int(actualsizes[0])) > 2 or \
                        abs(int(sizes[1]) - int(actualsizes[1])) > 2:
                    self.output.add_info('E', pkg, 'wrong-icon-size', fname,
                                         'expected:', 'x'.join(sizes), 'actual:', 'x'.join(actualsizes))



from Testing import get_tested_mock_package


NonLSBCompliancePackage = get_tested_mock_package(
    lazyload=True,
    name='fPing@1',
    files={
        '/usr/sbin/fping',
        '/usr/share/doc/packages/fPing',
        '/usr/share/doc/packages/fPing/CHANGELOG.md',
        '/usr/share/licenses/fPing',
        '/usr/share/licenses/fPing/COPYING',
        '/usr/share/man/man8/fping.8.gz',
    },
    header={
        'name': 'fPing@1',
        'version': '1.0_beta',
        'release': '1.0-beta'
    },
)



from mockdata.mock_menuxdg import (
    MenuXDGBadBinPackage,
    MenuXDGBadDupPackage,
    MenuXDGBadSecPackage,
    MenuXDGBadUTF8Package,
    MenuXDGInvalidPackage,
    MenuXDGPackage,
)
import pytest
from rpmlint.checks.MenuXDGCheck import MenuXDGCheck
from rpmlint.filter import Filter

from Testing import CONFIG, HAS_DESKTOP_FILE_UTILS


@pytest.fixture(scope='function', autouse=True)
def menuxdgcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = MenuXDGCheck(CONFIG, output)
    return output, test


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGInvalidPackage])
def test_raises_parse_error(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    assert len(output.results) == 4
    out = output.print_results(output.results)
    assert 'contains parsing error' in out
    assert ' invalid-desktopfile ' in out
    assert 'check with desktop-file-validate' in out


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGBadBinPackage])
def test_without_binary(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'desktopfile-without-binary' in out


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGBadDupPackage])
def test_duplicate(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'desktopfile-duplicate-section' in out
    assert 'invalid-desktopfile' in out


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGBadSecPackage])
def test_missing_header(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'desktopfile-missing-header' in out
    assert 'invalid-desktopfile' in out


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGBadUTF8Package])
def test_bad_unicode(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'non-utf8-desktopfile' in out


@pytest.mark.skipif(not HAS_DESKTOP_FILE_UTILS, reason='Optional dependency desktop-file-utils not installed')
@pytest.mark.parametrize('package', [MenuXDGPackage])
def test_good(package, menuxdgcheck):
    output, test = menuxdgcheck
    test.check(package)
    out = output.print_results(output.results)
    assert not out



from collections import defaultdict
import cProfile
import importlib
import operator
from pstats import Stats
import sys
from tempfile import gettempdir
import time

from rpmlint.color import Color
from rpmlint.config import Config
from rpmlint.filter import Filter
from rpmlint.helpers import print_warning, string_center
from rpmlint.pkg import FakePkg, get_installed_pkgs, Pkg
from rpmlint.version import __version__


class Lint:
    """
    Generic object handling the basic rpmlint operations
    """

    def __init__(self, options):
        # initialize configuration
        self.checks = {}
        self.options = options
        self.packages_checked = 0
        self.specfiles_checked = 0
        self.check_duration = defaultdict(int)
        if options['config']:
            self.config = Config(options['config'])
        else:
            self.config = Config()
        if options['profile']:
            self.profile = cProfile.Profile()
            self.profile.enable()
        else:
            self.profile = None

        if options['rpmlintrc']:
            options['rpmlintrc'] = [options['rpmlintrc']]
        self._load_rpmlintrc()
        if options['verbose']:
            self.config.info = options['verbose']
        if options['strict']:
            self.config.strict = options['strict']
        if options['permissive']:
            self.config.permissive = options['permissive']
        if not self.config.configuration['ExtractDir']:
            self.config.configuration['ExtractDir'] = gettempdir()
        # initialize output buffer
        self.output = Filter(self.config)
        # preload the check list if we not print config
        # some of the config values are transformed e.g. to regular
        # expressions
        if not self.options['print_config']:
            self.load_checks()

    def _run(self):
        start = time.monotonic()
        retcode = 0
        # if we just want to print config, do so and leave
        if self.options['print_config']:
            self.print_config()
            return retcode
        # just explain the error and abort too
        if self.options['explain']:
            self.print_explanation(self.options['explain'], self.config)
            return retcode
        # if there are installed arguments just load them up as extra
        # items to the rpmfile option
        if self.options['installed']:
            self.validate_installed_packages(self._load_installed_rpms(self.options['installed']))
        # if no exclusive option is passed then just loop over all the
        # arguments that are supposed to be either rpm or spec files
        self.validate_files(self.options['rpmfile'])
        self._print_header()
        print(self.output.print_results(self.output.results, self.config),
              end='')
        quit_color = Color.Bold
        if self.output.printed_messages['W'] > 0:
            quit_color = Color.Yellow
        if self.output.badness_threshold > 0 and self.output.score > self.output.badness_threshold:
            msg = string_center(f'Badness {self.output.score} exceeds threshold {self.output.badness_threshold}, aborting.', '-')
            print(f'{Color.Red}{msg}{Color.Reset}')
            quit_color = Color.Red
            retcode = 66
        elif self.output.printed_messages['E'] > 0 and not self.config.permissive:
            quit_color = Color.Red
            all_promoted = self.output.printed_messages['E'] == self.output.promoted_to_error
            retcode = 65 if all_promoted else 64

        self._maybe_print_reports()

        duration = time.monotonic() - start
        error_messages = self.output.printed_messages['E']
        warning_messages = self.output.printed_messages['W']
        msg = string_center(f'{self.packages_checked} packages and {self.specfiles_checked} specfiles checked; '
                            f'{error_messages} errors, {warning_messages} warnings'
                            f', {self.output.filtered_out} filtered, '
                            f'{self.output.score} badness; has taken {duration:.1f} s', '=')
        print(f'{quit_color}{msg}{Color.Reset}')

        return retcode

    def run(self):
        try:
            return self._run()
        except KeyboardInterrupt as e:
            self._maybe_print_reports()
            raise e

    def _maybe_print_reports(self):
        if self.options['time_report']:
            self._print_time_report()
        if self.profile:
            self._print_cprofile()

    def _get_color_time_report_value(self, fraction):
        if fraction > 25:
            color = Color.Red
        elif fraction > 5:
            color = Color.Yellow
        else:
            color = ''
        return f'{color}{fraction:17.1f}{Color.Reset}'

    def _print_time_report(self):
        PERCENT_THRESHOLD = 1
        TIME_THRESHOLD = 0.1
        total = sum(self.check_duration.values())
        checked_files = [check.checked_files for check in self.checks.values() if check.checked_files]
        total_checked_files = max(checked_files) if checked_files else ''
        print(f'{Color.Bold}Check time report{Color.Reset} (>{PERCENT_THRESHOLD}% & >{TIME_THRESHOLD}s):')

        check = format('Check', '32s')
        duration = format('Duration (in s)', '>12')
        fraction = format('Fraction (in %)', '>17')
        print(f'{Color.Bold}    {check} {duration} {fraction}  Checked files{Color.Reset}')

        for check, duration in sorted(self.check_duration.items(), key=operator.itemgetter(1), reverse=True):
            fraction = 100.0 * duration / total
            if fraction < PERCENT_THRESHOLD or duration < TIME_THRESHOLD:
                continue

            checked_files = ''
            if check in self.checks:
                checked = self.checks[check].checked_files
                if checked:
                    checked_files = checked
            print(f'    {check:32s} {duration:15.1f} {self._get_color_time_report_value(fraction)} {checked_files:>14}')

        print(f'    {"TOTAL":32s} {total:15.1f} {100:17.1f} {total_checked_files:>14}\n')       # noqa Q000

    def _print_cprofile(self):
        N = 30
        print(f'{Color.Bold}cProfile report:{Color.Reset}')
        self.profile.disable()
        stats = Stats(self.profile)
        stats.sort_stats('cumulative').print_stats(N)
        print('========================================================')
        stats.sort_stats('ncalls').print_stats(N)
        print('========================================================')
        stats.sort_stats('tottime').print_stats(N)

    def _load_installed_rpms(self, packages):
        existing_packages = []
        for name in packages:
            pkg = get_installed_pkgs(name)
            if pkg:
                existing_packages.extend(pkg)
            else:
                print_warning(f'(none): E: there is no installed rpm "{name}".')
        return existing_packages

    def _load_rpmlintrc(self):
        """
        Load rpmlintrc from argument or load up from folder
        """
        if self.options['rpmlintrc']:
            # Right now, we allow loading of just a single file, but the 'opensuse'
            # branch contains auto-loading mechanism that can eventually load
            # multiple files.
            for rcfile in self.options['rpmlintrc']:
                self.config.load_rpmlintrc(rcfile)
        else:
            # load only from the same folder specname.rpmlintrc or specname-rpmlintrc
            # do this only in a case where there is one folder parameter or one file
            # to avoid multiple folders handling
            rpmlintrc = []
            if len(self.options['rpmfile']) != 1:
                return
            pkg = self.options['rpmfile'][0]
            if pkg.is_file():
                pkg = pkg.parent
            rpmlintrc += sorted(pkg.glob('*.rpmlintrc'))
            rpmlintrc += sorted(pkg.glob('*-rpmlintrc'))
            if len(rpmlintrc) > 1:
                # multiple rpmlintrcs are highly undesirable
                print_warning('There are multiple items to be loaded for rpmlintrc, ignoring them: {}.'.format(' '.join(map(str, rpmlintrc))))
            elif len(rpmlintrc) == 1:
                self.options['rpmlintrc'] = rpmlintrc[0]
                self.config.load_rpmlintrc(rpmlintrc[0])

    def _print_header(self):
        """
        Print out header information about the state of the
        rpmlint prior printing out the check report.
        """
        intro = string_center('rpmlint session starts', '=')
        print(f'{Color.Bold}{intro}{Color.Reset}')
        print(f'rpmlint: {__version__}')
        print('configuration:')
        for config in self.config.conf_files:
            print(f'    {config}')
        if self.options['rpmlintrc']:
            rpmlintrc = self.options['rpmlintrc']
            print(f'rpmlintrc: {rpmlintrc}')
        no_checks = len(self.config.configuration['Checks'])
        no_pkgs = len(self.options['installed']) + len(self.options['rpmfile'])
        print(f'{Color.Bold}checks: {no_checks}, packages: {no_pkgs}{Color.Reset}')
        print('')

    def validate_installed_packages(self, packages):
        for pkg in packages:
            self.run_checks(pkg, pkg == packages[-1])
            self.reset_checks()

    def validate_files(self, files):
        """
        Run all the check for passed file list
        """
        if not files:
            if self.packages_checked == 0:
                # print warning only if we didn't process even installed files
                print_warning('There are no files to process nor additional arguments.')
                print_warning('Nothing to do, aborting.')
            return
        # check all elements if they are a folder or a file with proper suffix
        # and expand everything
        packages = self._expand_filelist(files)

        # Sort the files so that the output is stable
        packages = sorted(packages)
        for pkg in packages:
            self.validate_file(pkg, pkg == packages[-1])
            self.reset_checks()

    def _expand_filelist(self, files):
        packages = []
        for pkg in files:
            if pkg.is_file() and pkg.suffix in ('.rpm', '.spm', '.spec'):
                packages.append(pkg)
            elif pkg.is_dir():
                packages.extend(self._expand_filelist(pkg.iterdir()))
        return packages

    def validate_file(self, pname, is_last):
        try:
            if pname.suffix == '.rpm' or pname.suffix == '.spm':
                with Pkg(pname, self.config.configuration['ExtractDir'],
                         verbose=self.config.info) as pkg:
                    for k, v in pkg.timers.items():
                        self.check_duration[k] += v
                    self.run_checks(pkg, is_last)
            elif pname.suffix == '.spec':
                with FakePkg(pname) as pkg:
                    self.run_checks(pkg, is_last)
        except Exception as e:
            print_warning(f'(none): E: fatal error while reading {pname}: {e}')
            if self.config.info:
                raise e
            else:
                sys.exit(3)

    def run_checks(self, pkg, is_last):
        spec_checks = isinstance(pkg, FakePkg)
        for checker in self.checks:
            start = time.monotonic()
            fn = self.checks[checker].check_spec if spec_checks else self.checks[checker].check
            fn(pkg)
            self.check_duration[checker] += time.monotonic() - start

        # run post check function and validate used filters in rpmlintrc
        if is_last:
            for checker in self.checks.values():
                checker.after_checks()

            if not self.options['ignore_unused_rpmlintrc']:
                self.output.validate_filters(pkg)

        if spec_checks:
            self.specfiles_checked += 1
        else:
            self.packages_checked += 1

    def print_config(self):
        """
        Just output the current configuration
        """
        self.config.print_config()

    def print_explanation(self, messages, config):
        """
        Print out detailed explanation for the specified messages
        """
        for message in messages:
            explanation = self.output.get_description(message, config)
            if not explanation:
                # check if it's a WarnOnFunction warning configuration
                forbidden_functions = config.configuration['WarnOnFunction']
                if message in forbidden_functions:
                    explanation = forbidden_functions[message].get('description')

            if not explanation:
                explanation = 'Unknown message, please report a bug if the description should be present.\n\n'

            print(f'{message}:\n{explanation}')

    def load_checks(self):
        """
        Load all checks based on the config, skipping those already loaded
        SingletonTM
        """

        selected_checks = self.options['checks']
        if selected_checks:
            selected_checks = selected_checks.split(',')

        for check in self.config.configuration['Checks']:
            if check in self.checks:
                continue
            if not selected_checks or check in selected_checks:
                self.checks[check] = self.load_check(check)

    def reset_checks(self):
        """
        Reset all check objects to set to the default state
        """
        for check in self.checks.values():
            check.reset()

    def load_check(self, name):
        """Load a (check) module by its name, unless it is already loaded."""
        module = importlib.import_module(f'.{name}', package='rpmlint.checks')
        klass = getattr(module, name)
        obj = klass(self.config, self.output)
        return obj



from mockdata.mock_duplicates import (
    HardlinksAndDuplicatesPresent,
    NoHardLinksAndDuplicatesPresent
)
import pytest
from rpmlint.checks.DuplicatesCheck import DuplicatesCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def duplicatescheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = DuplicatesCheck(CONFIG, output)
    yield output, test


@pytest.fixture()
def test(duplicatescheck):
    _output, test = duplicatescheck
    yield test


@pytest.fixture()
def output(duplicatescheck):
    output, _test = duplicatescheck
    yield output


@pytest.mark.parametrize('package', [HardlinksAndDuplicatesPresent])
def test_duplicates1(package, test, output):
    test.check(package)
    out = output.print_results(output.results)

    assert 'E: hardlink-across-partition /var/foo /etc/foo' in out
    assert 'E: hardlink-across-config-files /var/foo2 /etc/foo2' in out
    assert 'W: files-duplicate /etc/bar3 /etc/bar:/etc/bar2' in out
    assert 'W: files-duplicate /etc/strace2.txt /etc/strace1.txt' in out
    assert 'W: files-duplicate /etc/small2 /etc/small' not in out
    assert 'E: files-duplicated-waste 270543' in out


@pytest.mark.parametrize('package', [NoHardLinksAndDuplicatesPresent])
def test_duplicates_correct(package, test, output):
    test.check(package)
    out = output.print_results(output.results)

    assert 'E: hardlink-across-partition' not in out
    assert 'E: hardlink-across-config-files' not in out
    assert 'W: files-duplicate' not in out
    assert 'E: files-duplicated-waste' not in out



# coding=utf-8
"""
Provides crossplatform checking of current power source, battery warning level and battery time remaining estimate.
Allows you to add observer for power notifications if platform supports it.

Usage:
    from power import PowerManagement, PowerManagementObserver # Automatically imports platform-specific implementation

    class Observer(PowerManagementObserver):
        def on_power_sources_change(self, power_management):
            print "Power sources did change."

        def on_time_remaining_change(self, power_management):
            print "Time remaining did change."

    # class Observer(object):
    #     ...
    # PowerManagementObserver.register(Observer)
"""
__author__ = 'kulakov.ilya@gmail.com'
__version__ = '1.1'

from sys import platform
from power.common import *


try:
    if platform.startswith('darwin'):
        from power.darwin import PowerManagement
    elif platform.startswith('win32'):
        from power.win32 import PowerManagement
    elif platform.startswith('linux'):
        from power.linux import PowerManagement
    else:
        raise RuntimeError("{platform} is not supported.".format(platform=platform))
except RuntimeError as e:
    import warnings
    warnings.warn("Unable to load PowerManagement for {platform}. No-op PowerManagement class is used: {error}".format(error=str(e), platform=platform))
    from power.common import PowerManagementNoop as PowerManagement



from Testing import get_tested_mock_package


BuildrootPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/bin/trace': {'content': '/home/marxin/rpmbuild/BUILDROOT/buildroot-0-0.x86_64'},
    },
)



from pathlib import Path
import re
import stat

import rpm
from rpmlint.checks.AbstractCheck import AbstractCheck
from rpmlint.pkg import formatRequire
from rpmlint.readelfparser import ReadelfParser


class SharedLibraryPolicyCheck(AbstractCheck):
    """
    Verify shared library packaging policy rules

    This package validates the shared libraries naming scheme based
    on the Debian/openSUSE shared libraries policy:
     https://en.opensuse.org/openSUSE:Shared_library_packaging_policy
     https://www.debian.org/doc/debian-policy/ch-sharedlibs.html
    """

    def __init__(self, config, output):
        super().__init__(config, output)
        self.re_soname_strongly_versioned = re.compile(r'-[\d\.]+\.so$')
        # the pkgname is based on soname if ending with number; special option is flavor build
        self.re_soname_pkg = re.compile(r'^lib\S+(\d+(-(32|64)bit)?)$')
        self.re_so_files = re.compile(r'\S+.so((\.(\d+))*)$')

    def _check_missing_policy_lib(self, pkg):
        # check the pkg has any libname
        libfiles = []
        for fname in pkg.files.keys():
            if not self.re_so_files.match(fname):
                continue
            libfiles.append(fname)

        # if we didn't find any library files then we
        # don't need to check anything

        # verify if name does not match the slpp and if we still don't have any lib then error out
        if not libfiles and self.re_soname_pkg.match(pkg.name):
            self.output.add_info('E', pkg, 'shlib-policy-missing-lib')

    def check(self, pkg):
        if pkg.is_source:
            return

        # Consider only non-development, non-language library packages
        if (not pkg.name.startswith('lib') or
                pkg.name.endswith('-devel') or
                pkg.name.endswith('-lang')):
            return

        self._check_missing_policy_lib(pkg)

        # the soname validation matching the name is done
        # already in BinaryCheck._check_shared_library
        # Search for shared libraries in this package
        libs = set()
        libs_needed = set()
        libs_to_dir = {}
        reqlibs = set()
        pkg_requires = {x.name.split('(')[0] for x in pkg.requires}

        for filename, pkgfile in pkg.files.items():
            path = Path(filename)
            if (('.so.' in filename or filename.endswith('.so')) and
                    stat.S_ISREG(pkg.files[filename].mode) and pkgfile.magic.startswith('ELF ')):
                readelf_parser = ReadelfParser(pkgfile.path, filename)
                failed_reason = readelf_parser.parsing_failed_reason()
                if failed_reason:
                    self.output.add_info('E', pkg, 'readelf-failed', filename, failed_reason)
                    return
                dyn_section = readelf_parser.dynamic_section_info
                libs_needed = libs_needed.union(dyn_section.needed)
                if dyn_section.soname:
                    lib_dir = str(path.parent)
                    libs.add(dyn_section.soname)
                    libs_to_dir[dyn_section.soname] = lib_dir
                if dyn_section.soname in pkg_requires:
                    # But not if the library is used by the pkg itself
                    # This avoids program packages with their own
                    # private lib
                    # FIXME: we'd need to check if somebody else links
                    # to this lib
                    reqlibs.add(dyn_section.soname)

        if not libs.difference(reqlibs):
            return

        if pkg.name[-1].isdigit():
            # ignore libs in a versioned non_std_dir
            for lib in libs.copy():
                lib_dir = libs_to_dir[lib]
                for lib_part in lib_dir.split('/'):
                    if not lib_part:
                        continue
                    if lib_part[-1].isdigit() and not lib_part.endswith('lib64'):
                        libs.remove(lib)
                        break

            # Check for non-versioned libs in a std lib package
            for lib in libs.copy():
                if (not (lib[-1].isdigit() or
                         self.re_soname_strongly_versioned.search(lib))):
                    self.output.add_info('W', pkg, 'shlib-unversioned-lib', lib)

            # Verify shared lib policy package doesn't have hard dependency on non-lib packages
            for dep in pkg.requires:
                if dep[0].startswith('rpmlib(') or dep[0].startswith('config('):
                    continue
                if (dep[1] & (rpm.RPMSENSE_GREATER | rpm.RPMSENSE_EQUAL)) == rpm.RPMSENSE_EQUAL:
                    self.output.add_info('W', pkg, 'shlib-fixed-dependency', formatRequire(dep[0], dep[1], dep[2]))

        # Verify non-lib stuff does not add dependencies
        if libs:
            for dep in pkg_requires:
                if '.so.' in dep and dep not in libs and dep not in libs_needed:
                    self.output.add_info('E', pkg, 'shlib-policy-excessive-dependency', dep)

        # FIXME: implement (#567)
        # Check if the files/folders are unversioned in the library package.
        # In general you can't co-install the soname packages if they all provide some datafiles
        # or configuration files.
        # When testing one of examples is libsemanage1:
        #   /etc/selinux
        #   /etc/selinux/semanage.conf
        #   /usr/lib64/libsemanage.so.1
        # The above would be fine if the semanage.conf would be update-alternatived, or suffixed
        # but if someone introduces libsemanage2 they can't be installed both at once.



import stat

from Testing import get_tested_mock_package


LogrotatePackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/tmp/foo': {
            'create_dirs': True,
            'is_dir': True,
            'metadata': {
                'mode': stat.S_IFDIR | 0o755,
                'user': 'marxin',
                'group': 'users',
            },
        },
        '/tmp/foo2': {
            'create_dirs': True,
            'is_dir': True,
            'metadata': {
                'mode': stat.S_IFREG | stat.S_ISUID | 0o777,
                'user': 'root',
                'group': 'users2',
            },
        },
        '/tmp/foo/my.log': {
            'metadata': {
                'mode': 0o644 | stat.S_IFREG,
                'user': 'root',
                'group': 'users2',
            },
        },
        '/tmp/foo2/my.log': {
            'metadata': {
                'mode': stat.S_IFREG | stat.S_ISUID | 0o777,
                'user': 'root',
                'group': 'users2',
            },
        },
        '/etc/logrotate.d/logrotate.conf': {
            'create_dirs': True,
            'content-path': 'files/logrotate/logrotate.conf',
        },
        '/etc/logrotate.d/logrotate2.conf': {
            'create_dirs': True,
            'content-path': 'files/logrotate/logrotate2.conf',
        },
    }
)



from Testing import get_tested_mock_package


BROKEN = """
[DEFAULT]
something = true
[else
"""

BADBIN = """
[Desktop Entry]
Name=rpmlint-test
Exec=rpmlint-test file.file
Icon=chameleon_v_balíku
Type=Application
GenericName=rpmlint testcase
Categories=Game;Amusement;
"""


BADDUP = """
[Desktop Entry]
[Desktop Entry]
Name=rpmlint-test
Name=name duplicate
Exec=rpmlint-test file.file
Icon=chameleon_v_balíku
Type=Application
GenericName=rpmlint testcase
Categories=Game;Amusement;
"""


BADSEC = """
Name=rpmlint-test
[Bad Section]
Name[x-test]=xxrpmlint-testxx
Exec=rpmlint-test file.file
Icon=rpmlint-test
Type=Application
GenericName=rpmlint testcase
"""


MenuXDGInvalidPackage = get_tested_mock_package(
    lazyload=True,
    files={'/usr/share/applications/somefile.desktop': {'content': BROKEN}},
)


MenuXDGBadBinPackage = get_tested_mock_package(
    lazyload=True,
    files={'/usr/share/applications/rpmlint-test.desktop': {'content': BADBIN}},
)


MenuXDGBadDupPackage = get_tested_mock_package(
    lazyload=True,
    files={'/usr/share/applications/rpmlint-test.desktop': {'content': BADDUP}},
)


MenuXDGBadSecPackage = get_tested_mock_package(
    lazyload=True,
    files={'/usr/share/applications/rpmlint-test.desktop': {'content': BADSEC}},
)


MenuXDGBadUTF8Package = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/share/applications/rpmlint-test.desktop': {
            'content-path': 'files/rpmlint-test.desktop',
            'create_dirs': True,
        },
    },
)

MenuXDGPackage = get_tested_mock_package(
    lazyload=True,
    files={
        '/usr/bin/rpmlint-test': {},
        '/usr/share/applications/rpmlint-test.desktop': {
            'content-path': 'files/rpmlint-test-good.desktop',
            'create_dirs': True,
        },
    },
)



from os.path import basename
from pathlib import Path
import re
import stat

import rpm
from rpmlint.checks.AbstractCheck import AbstractCheck
from rpmlint.helpers import byte_to_string


class AlternativesCheck(AbstractCheck):
    """
    Check for compliance with update-alternatives usage guidelines:
      http://en.opensuse.org/openSUSE:Packaging_Multiple_Version_guidelines
      https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
    In short the rules are:
      /etc/alternative/basename must be in %files and must be %ghost file
      The alternative master must be a symlink to /etc and in filelist. I.e.:
        /usr/bin/basename -> /etc/alternative/basename
      In %post the update-alternatives with install must be called
      In %postun the update-alternatives with remove must be called
      Requires(post) and Requires(postun) must depend on update-alternatives
    """
    # Regex to match anything that can be in requires for update-alternatives
    re_requirement = re.compile(r'^(/usr/sbin/|%{?_sbindir}?/)?update-alternatives$')
    re_install = re.compile(r'--install\s+(?P<link>\S+)\s+(?P<name>\S+)\s+(\S+)\s+(\S+)')
    re_slave = re.compile(r'--slave\s+(?P<link>\S+)\s+(\S+)\s+(\S+)')
    command = 'update-alternatives'
    alts_requirement = 'alts'

    def __init__(self, config, output):
        super().__init__(config, output)

    def check(self, pkg):
        if pkg.is_source:
            return

        if self._check_libalternatives_presence(pkg):
            self.output.add_info('I', pkg, 'package-supports-libalternatives')
            self._check_libalternatives_requirements(pkg)
            self._check_libalternatives_filelist(pkg)

        self.install_binaries = {}
        self.slave_binaries = []
        # populate scriptlets
        self.post = byte_to_string(pkg.header[rpm.RPMTAG_POSTIN])
        self.postun = byte_to_string(pkg.header[rpm.RPMTAG_POSTUN])

        if not self._check_ua_presence(pkg):
            return
        self.output.add_info('I', pkg, 'package-supports-update-alternatives')

        self._check_requirements(pkg)
        self._check_post_phase(pkg, self.post)
        self._check_postun_phase(pkg, self.postun)
        self._check_filelist(pkg)

    def _find_u_a_binarires(self, line):
        """
        Find all binaries that have install or slave that are needed
        to be validated.
        update-alternatives --install link name path priority [--slave link name path]+
        """
        match = self.re_install.search(line)
        if match:
            self.install_binaries[match.group('link')] = match.group('name')
        # --slave can be repeated multiple times
        matches = self.re_slave.finditer(line)
        for match in matches:
            self.slave_binaries.append(match.group('link'))

    def _check_post_phase(self, pkg, script):
        """
        Validate that post phase contains the update-alternatives --install call
        Collect all binaries that are to be validated for the usage
        """
        script = self._normalize_script(script)
        # If there is no u-a call then give up right away
        if not script:
            self.output.add_info('E', pkg, 'update-alternatives-post-call-missing')
            return
        # collect all the known binaries
        for line in script:
            self._find_u_a_binarires(line)
        # if there is u-a call, but no --install command it is still an issue
        if not self.install_binaries:
            self.output.add_info('E', pkg, 'update-alternatives-post-call-missing')

    def _check_postun_phase(self, pkg, script):
        """
        Validate that post phase contains the update-alternatives --remove call
        Make sure there is --remove line for all installed binaries
        update-alternatives --remove name path
        """
        script = self._normalize_script(script)
        # If there is no u-a call then give up right away
        if not script:
            self.output.add_info('E', pkg, 'update-alternatives-postun-call-missing')
            return
        # validate each binary actually is properly removed
        binaries = list(self.install_binaries.values())
        # we remove from the binaries list in the loop, copy it
        for binary in binaries.copy():
            re_remove = re.compile(fr'--remove\s+{re.escape(binary)}\b')
            for line in script:
                if re_remove.search(line) and binary in binaries:
                    binaries.remove(binary)
        for binary in binaries:
            self.output.add_info('E', pkg, 'update-alternatives-postun-call-missing', binary)

    def _check_filelist(self, pkg):
        """
        Validate all filelists for required content to make u-a work:
        * For each install/slave binary I need /etc/alternatives/X
          + This file must be in filelist marked as ghost
        * The install/slave binary must be present in filelist
          + The item must be a a link to /etc/alternatives
        """
        files = pkg.files
        ghost_files = pkg.ghost_files
        for binary in self.slave_binaries + list(self.install_binaries.keys()):
            etc_alt_file = '/etc/alternatives/%s' % basename(binary)
            if etc_alt_file not in files:
                # The alternative is missing completely
                self.output.add_info('E', pkg, 'alternative-link-missing', etc_alt_file)
            elif etc_alt_file not in ghost_files:
                # The alternative is present, but not as ghost
                self.output.add_info('E', pkg, 'alternative-link-not-ghost', etc_alt_file)

            # generic-name should be a symlink to /etc/alternatives/$(basename)
            if binary not in files:
                self.output.add_info('E', pkg, 'alternative-generic-name-missing', binary)
            elif not stat.S_ISLNK(files[binary].mode):
                self.output.add_info('E', pkg, 'alternative-generic-name-not-symlink', binary)

    def _check_ua_presence(self, pkg):
        """
        Check if there is update-alternatives scriptlet present and if we should do validation
        """
        # first check just if we have anything in /etc/alternatives
        for path in pkg.files:
            if path.startswith('/etc/alternatives'):
                return True
        # then check the scriptlets if they run update-alternatives
        if self._check_scriptlet_for_alternatives(self.post):
            return True
        if self._check_scriptlet_for_alternatives(self.postun):
            return True
        return False

    def _check_libalternatives_presence(self, pkg):
        """
        Check if there is libalternatives scriptlet present
        """
        # first check just if we have anything in /usr/share/libalternatives/
        for path in pkg.files:
            if path.startswith('/usr/share/libalternatives/'):
                return True
        # then check if package with the name "alts" is required
        return any(req[0] == self.alts_requirement for req in pkg.requires + pkg.prereq)

    def _check_scriptlet_for_alternatives(self, scriptlet):
        """
        Check if scriptlet actually contains the update-alternatives call
        """
        if scriptlet is not None and self.command in scriptlet:
            return True
        return False

    def _normalize_script(self, script):
        """
        Remove "backslash+newline" to keep all commands as oneliners.
        Remove single and double quotes everywhere.
        Keep only the line that contains the update-alternatives call.
        Return the list of lines that contain update-alternatives calls
        """
        # with old rpm we get wrong type
        script = byte_to_string(script)
        if script is None:
            return None
        script = script.replace('\\\n', '')
        script = script.replace('"', '')
        script = script.replace("'", '')
        script = script.strip()
        return [i for i in script.splitlines() if self.command in i]

    def _check_requirements(self, pkg):
        """
        Check that Requires(post/postun) contain the update-alternatives dependency
        """
        for require in pkg.prereq:
            if self.re_requirement.match(require[0]):
                return
        self.output.add_info('E', pkg, 'update-alternatives-requirement-missing')

    def _check_libalternatives_requirements(self, pkg):
        """
        Check the requirement of package "alts"
        """
        for req in pkg.requires + pkg.prereq:
            if req[0] == self.alts_requirement:
                return
        self.output.add_info('E', pkg, 'alts-requirement-missed')

    def _check_libalternatives_filelist(self, pkg):
        """
        Checking if all links to "alts" have corresponding entries in
        /usr/share/libalternatives.
        """
        for f, pkgfile in pkg.files.items():
            if pkgfile.linkto == Path(self.alts_requirement).name:
                dir_name = '/usr/share/libalternatives/' + Path(f).name
                if dir_name not in pkg.files:
                    self.output.add_info('E', pkg, 'libalternatives-directory-not-exists', dir_name)
                else:
                    r = re.compile('^' + dir_name + '/.*.conf$')
                    if not list(filter(r.match, pkg.files)):
                        self.output.add_info('E', pkg, 'empty-libalternatives-directory', dir_name)
        """
        Checking content of all /usr/share/libalternatives/*/*.conf files
        """
        for f, pkgfile in pkg.files.items():
            if re.search('^/usr/share/libalternatives/.*conf$', f):
                filename = Path(pkg.dirname + f)
                if not filename.exists():
                    if pkgfile.is_ghost:
                        self.output.add_info('I', pkg, 'libalternatives-conf-not-found', f)
                    else:
                        self.output.add_info('E', pkg, 'libalternatives-conf-not-found', f)
                    continue
                bin_found = False
                man_found = False
                with open(filename) as read_obj:
                    # Read all lines in the file one by one. E.g:
                    #
                    # binary=/usr/bin/jupyter-3.8
                    # man=jupyter-3.8.1
                    # group=jupyter, jupyter-migrate, jupyter-troubleshoot
                    #
                    for line_nr, line in enumerate(read_obj):
                        line_array = [x.strip() for x in line.split('=')]
                        line_nr_str = f'Line: {line_nr}'
                        if len(line_array) != 2:   # empty values are valid
                            self.output.add_info('E', pkg, 'wrong-entry-format', f, line_nr_str)

                        key, value = line_array
                        if key == 'binary':
                            if bin_found:
                                self.output.add_info('E', pkg, 'multiple-entries', f, line_nr_str)
                                continue
                            for path in pkg.files:
                                if 'bin/' in path and path.endswith(value):
                                    bin_found = True
                            if not bin_found:
                                self.output.add_info('W', pkg, 'binary-entry-value-not-found', f, line_nr_str)
                        elif key == 'man':
                            if man_found:
                                self.output.add_info('E', pkg, 'double-entries', f, line_nr_str)
                                continue
                            mans = value.split(',')
                            for man in mans:
                                man_found = False
                                for path in pkg.files:
                                    if path.startswith('/usr/share/man/') and man.strip() in path:
                                        man_found = True
                                if not man_found:
                                    self.output.add_info('W', pkg, 'man-entry-value-not-found', f, line_nr_str)
                        elif key != 'group' and key != 'options':
                            self.output.add_info('W', pkg, 'wrong-tag-found', f, line_nr_str)
                    if not bin_found:
                        self.output.add_info('W', pkg, 'wrong-or-missed-binary-entry', f)



from mockdata.mock_pkgconfig import LibReiserFSCoreDevelPackage, PCPackage
import pytest
from rpmlint.checks.PkgConfigCheck import PkgConfigCheck
from rpmlint.filter import Filter

from Testing import CONFIG


@pytest.fixture(scope='function', autouse=True)
def pkgconfigcheck():
    CONFIG.info = True
    output = Filter(CONFIG)
    test = PkgConfigCheck(CONFIG, output)
    return output, test


@pytest.mark.parametrize('package', [PCPackage])
def test_pkg_config(package, pkgconfigcheck):
    output, test = pkgconfigcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: invalid-pkgconfig-file /tmp/pkgconfig/xcb.pc' in out
    assert 'E: pkgconfig-invalid-libs-dir /tmp/pkgconfig/xcb.pc Libs: -L/usr/lib' in out
    assert 'E: double-slash-in-pkgconfig-path /tmp/pkgconfig/xcb.pc includedir=/usr/include//xyz' in out


@pytest.mark.parametrize('package', [LibReiserFSCoreDevelPackage])
def test_pkg_config_correct(package, pkgconfigcheck):
    output, test = pkgconfigcheck
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: invalid-pkgconfig-file' not in out
    assert 'E: pkgconfig-invalid-libs-dir' not in out
    assert 'E: double-slash-in-pkgconfig-path' not in out



from rpmlint import helpers


def test_warnprint(capsys):
    """
    Check we print stuff to stderr
    """
    message = 'I am writing to stderr'
    helpers.print_warning(message)
    out, err = capsys.readouterr()
    assert message not in out
    assert message in err


def test_bytetostr():
    """
    Test bytetostr function
    """
    list_items = (
        b'\xc5\xbe\xc3\xad\xc5\xbeala',
        'texty',
    )
    item = b'p\xc5\x99\xc3\xad\xc5\xa1ern\xc4\x9b \xc5\xbelu\xc5\xa5ou\xc4\x8dk\xc3\xbd k\xc5\xaf\xc5\x88'

    result = helpers.byte_to_string(item)
    assert isinstance(result, str)
    assert result == 'příšerně žluťoučký kůň'

    result = helpers.byte_to_string(list_items)
    assert isinstance(result, list)
    assert result[0] == 'žížala'


def test_centering(capsys):
    """
    Check wether centered print works
    """

    message = 'Hello there'
    helpers.print_centered(message, '*')
    out, err = capsys.readouterr()
    assert '** Hello there **' in out
    assert not err
